==> amazon_reports_downloader-0.0.45-py2.py3-none-any.whl: amazon_reports_downloader/bin/get_code.py <==
# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import asyncio
import os
import sys
import logging
import logging.handlers
import requests
import re
import dropbox

import pkg_resources
from cmutils.config_loaders import IniConfigLoader


import click
import sentry_sdk

from amazon_reports_downloader.models import init_db, Store


from amazon_reports_downloader.config_loaders import YamlConfigLoader
from amazon_reports_downloader.exchange_rate import EcbExchangeRate

from amazon_reports_downloader.lib.google.sheet_api import SheetAPI
from amazon_reports_downloader import logger
from amazon_reports_downloader.disburse_task_v2 import DisburseTaskV2
import datetime
import time
import pytz
import cv2
import pyotp

chicagoTz = pytz.timezone("America/Chicago") 
sheet_api = SheetAPI()

def get_database_config(config_path):
    if not config_path:
        logger.error('config_path is required to run!')
        sys.exit(1)

    config_path = os.path.abspath(os.path.expanduser(config_path))
    if not os.path.isfile(config_path):
        logger.error('Could not find configuration file - %s', config_path)
        sys.exit(0)

    cl = YamlConfigLoader(config_path)
    config = cl.load()

    if 'account' not in config or not config['account']:
        logger.error('Could not find "account" configuration!')
        sys.exit(0)

    host = config['db']['host']
    user = str(config['db']['user'])
    password = config['account']['password']

    local_config_file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "config_local.ini")

    if not pkg_resources.resource_exists('lib', 'config/config.ini'):
        logger.error('ConfigurationFileNotFound!')
        return

    if local_config_file_path and os.path.isfile(local_config_file_path):
        config_path = local_config_file_path
    else:
        config_path = pkg_resources.resource_filename('lib', 'config/config.ini')

    log_dir = os.path.join(os.path.expanduser('~'), '.spree-tools', 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)
    log_path = os.path.join(log_dir, 'spree_product_downloader_{}.log'.format(site))
    level = logging.INFO
    max_bytes = 20 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    init_db(config_path)

    config = IniConfigLoader(config_path)
    store = None
    while True:
        try:
            store = Store.get_store(site)
            break
        except peewee.OperationalError as e:
            logger.exception(e)

            time.sleep(3)
        except Exception as e:
            logger.exception(e)
            break

    if store is None:
        logger.error('[StoreNotFound] %s', site)
        return

def get_code(account, force=False):
    if sys.platform.startswith('win'):
        work_dir = 'C:\\AmazonReportDownloader'
    else:
        work_dir = os.path.join(os.path.expanduser('~'), '.AmazonReportDownloader')

    log_dir = os.path.join(work_dir, 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)

    log_path = os.path.join(log_dir, 'disburse.log')
    level = logging.INFO
    max_bytes = 100 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    if not account:
        logger.error('account is required to run!')
        sys.exit(1)

    image_path = get_qc_code_image(account, force)
    code_uri = extract_otp_code_uri_from_qr(image_path)
    code = generate_otp_from_uri(code_uri)
    print(code)
    return code

def generate_otp_from_uri(totp_uri):
    if not totp_uri:
        return None

    retry_left_count = 5
    while retry_left_count > 0:
        try:
            totp = pyotp.parse_uri(totp_uri)
            current_otp = totp.now()
            return current_otp
        except pyotp.PyOTPException as e:
            print(f"Error parsing URI: {e}")
        except pyotp.otp.TotpError as e:
            print(f"Error generating TOTP: {e}")
        except Exception as e:
            print(f"Unexpected error: {e}")
        finally:
            retry_left_count -= 1

def extract_otp_code_uri_from_qr(image_path):
    if not image_path:
        return None

    try:
        image = cv2.imread(image_path)
        qr_code_decoder = cv2.QRCodeDetector()
        retval, decoded_info, points, straight_qrcode = qr_code_decoder.detectAndDecodeMulti(image)
        if not retval:
            return None

        for info in decoded_info:
            if 'otpauth' in info:
                return info

    except Exception as e:
        pass

def get_qr_code_image_url(account):
    sheet_name = "code"
    sheet_id = "1b8wHMP05Na4ELcyP5Jn9syb9g6uMc8FkBTsckOCIz7w"
    worksheet = sheet_api.create_new_sheet_if_not_existed(sheet_id, sheet_name=sheet_name)
    key = account
    cell = sheet_api.find_by_key(worksheet, key)
    row = cell.row
    url_column = 14
    return sheet_api.cell_value(worksheet, row, url_column)


def get_image_type(response):
    type = None
    type_str = response.headers["content-type"]
    if type_str:
        type = type_str.split("/").pop()
    return type

def get_qc_code_image(account, force=False):
    image_url = get_qr_code_image_url(account)
    response = requests.get(image_url, stream=True)
    if response.headers["content-type"].find("text/html") > -1:
        image_url = change_image_url(image_url)

    if not image_url:
        return

    try:
        response = requests.get(image_url, stream=True)
        type = get_image_type(response)
        save_path = os.path.join(os.path.expanduser('~'), '.amazon_seller_management', f'{account}.{type}')
        if os.path.exists(save_path) and not force:
            return save_path
        remove_qc_code_image(save_path)
        if response.status_code == 200:
            save_dir = os.path.dirname(save_path)
            
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)

            with open(save_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=128):
                    file.write(chunk)
            
            return save_path
        else:
            print(f"Failed to download image. Status code: {response.status_code}")

    except Exception as e:
        print(f"Error downloading image: {e}")

def change_image_url(image_url):
    if not image_url:
        return None
    if image_url.find("google") > -1:
        return change_google_image_url(image_url)
    elif image_url.find("dropbox") > -1:
        return None
        return change_dropbox_image_url(image_url)
    else:
        raise ValueError("Unsupported image URL")

def change_google_image_url(image_url):
    match = re.search(r'/file/d/([a-zA-Z0-9_-]+)', image_url)
    if match:
        file_id = match.group(1)
        return f'https://drive.google.com/uc?id={file_id}&export=view&authuser=0'
    else:
        return None

def change_dropbox_image_url(image_url):
    direct_link = image_url.replace("www.dropbox.com", "dl.dropboxusercontent.com")
    if '?' in direct_link:
        direct_link = direct_link.split('?')[0]
    return direct_link

def remove_qc_code_image(save_path):
    try:
        os.remove(save_path)
        print(f"File {save_path} deleted successfully.")
    except FileNotFoundError:
        print(f"File {save_path} not found.")
    except Exception as e:
        print(f"Error deleting file: {e}")

if __name__ == '__main__':
    account = sys.argv[1]
    get_code(account)

==> amazon_reports_downloader-0.0.49-py2.py3-none-any.whl: amazon_reports_downloader/bin/get_code.py <==
# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import asyncio
import os
import sys
import logging
import logging.handlers
import requests
import re
import dropbox

import pkg_resources
from cmutils.config_loaders import IniConfigLoader


import click
import sentry_sdk

from amazon_reports_downloader.models import init_db, Store


from amazon_reports_downloader.config_loaders import YamlConfigLoader
from amazon_reports_downloader.exchange_rate import EcbExchangeRate

from amazon_reports_downloader.lib.google.sheet_api import SheetAPI
from amazon_reports_downloader import logger
from amazon_reports_downloader.disburse_task_v2 import DisburseTaskV2
import datetime
import time
import pytz
import cv2
import pyotp

chicagoTz = pytz.timezone("America/Chicago") 
sheet_api = SheetAPI()

def get_database_config(config_path):
    if not config_path:
        logger.error('config_path is required to run!')
        sys.exit(1)

    config_path = os.path.abspath(os.path.expanduser(config_path))
    if not os.path.isfile(config_path):
        logger.error('Could not find configuration file - %s', config_path)
        sys.exit(0)

    cl = YamlConfigLoader(config_path)
    config = cl.load()

    if 'account' not in config or not config['account']:
        logger.error('Could not find "account" configuration!')
        sys.exit(0)

    host = config['db']['host']
    user = str(config['db']['user'])
    password = config['account']['password']

    local_config_file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "config_local.ini")

    if not pkg_resources.resource_exists('lib', 'config/config.ini'):
        logger.error('ConfigurationFileNotFound!')
        return

    if local_config_file_path and os.path.isfile(local_config_file_path):
        config_path = local_config_file_path
    else:
        config_path = pkg_resources.resource_filename('lib', 'config/config.ini')

    log_dir = os.path.join(os.path.expanduser('~'), '.spree-tools', 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)
    log_path = os.path.join(log_dir, 'spree_product_downloader_{}.log'.format(site))
    level = logging.INFO
    max_bytes = 20 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    init_db(config_path)

    config = IniConfigLoader(config_path)
    store = None
    while True:
        try:
            store = Store.get_store(site)
            break
        except peewee.OperationalError as e:
            logger.exception(e)

            time.sleep(3)
        except Exception as e:
            logger.exception(e)
            break

    if store is None:
        logger.error('[StoreNotFound] %s', site)
        return

def get_code(account, force=False, image_url=''):
    if sys.platform.startswith('win'):
        work_dir = 'C:\\AmazonReportDownloader'
    else:
        work_dir = os.path.join(os.path.expanduser('~'), '.AmazonReportDownloader')

    log_dir = os.path.join(work_dir, 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)

    log_path = os.path.join(log_dir, 'disburse.log')
    level = logging.INFO
    max_bytes = 100 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    if not account:
        logger.error('account is required to run!')
        sys.exit(1)

    image_path = get_qc_code_image(account, force, image_url)
    code_uri = extract_otp_code_uri_from_qr(image_path)
    code = generate_otp_from_uri(code_uri)
    print(code)
    return code

def generate_otp_from_uri(totp_uri):
    if not totp_uri:
        return None

    retry_left_count = 5
    while retry_left_count > 0:
        try:
            totp = pyotp.parse_uri(totp_uri)
            current_otp = totp.now()
            return current_otp
        except Exception as e:
            print(f"Unexpected error: {e}")
        finally:
            retry_left_count -= 1

def extract_otp_code_uri_from_qr(image_path):
    if not image_path:
        return None

    try:
        image = cv2.imread(image_path)
        qr_code_decoder = cv2.QRCodeDetector()
        retval, decoded_info, points, straight_qrcode = qr_code_decoder.detectAndDecodeMulti(image)
        if not retval:
            return None

        for info in decoded_info:
            if 'otpauth' in info:
                return info

    except Exception as e:
        pass

def get_qr_code_image_url(account):
    sheet_name = "code"
    sheet_id = "1b8wHMP05Na4ELcyP5Jn9syb9g6uMc8FkBTsckOCIz7w"
    worksheet = sheet_api.create_new_sheet_if_not_existed(sheet_id, sheet_name=sheet_name)
    key = account
    cell = sheet_api.find_by_key(worksheet, key)
    row = cell.row
    url_column = 14
    return sheet_api.cell_value(worksheet, row, url_column)

def get_image_type(response):
    type_str = response.headers["content-type"]

    return response.headers["Content-Disposition"].split(".").pop() if type_str == "application/binary" else type_str.split("/").pop()

def get_qc_code_image(account, force=False, image_url=''):
    image_url = get_qr_code_image_url(account) if image_url == '' else image_url
    if not image_url:
        return

    response = requests.get(image_url, stream=True)
    if response.headers["content-type"].find("text/html") > -1:
        image_url = change_image_url(image_url)

    if not image_url:
        return

    try:
        response = requests.get(image_url, stream=True)
        type = get_image_type(response)
        save_path = os.path.join(os.path.expanduser('~'), '.amazon_seller_management', f'{account}.{type}')
        if os.path.exists(save_path) and not force:
            return save_path

        remove_qc_code_image(save_path)
        if response.status_code == 200:
            save_dir = os.path.dirname(save_path)
            
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)

            with open(save_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=128):
                    file.write(chunk)
            
            return save_path
        else:
            print(f"Failed to download image. Status code: {response.status_code}")

    except Exception as e:
        print(f"Error downloading image: {e}")

def change_image_url(image_url):
    if not image_url:
        return None
    if image_url.find("google") > -1:
        return change_google_image_url(image_url)
    elif image_url.find("dropbox") > -1:
        return change_dropbox_image_url(image_url)
    else:
        raise ValueError("Unsupported image URL")

def change_google_image_url(image_url):
    match = re.search(r'/file/d/([a-zA-Z0-9_-]+)', image_url)
    if match:
        file_id = match.group(1)
        return f'https://drive.google.com/uc?id={file_id}&export=view&authuser=0'
    else:
        return None

def change_dropbox_image_url(url):
    if "dl=0" not in url:
        return url
    modified_url = re.sub(r'dl=0', 'dl=1', url)
    return modified_url

def remove_qc_code_image(save_path):
    try:
        os.remove(save_path)
        print(f"File {save_path} deleted successfully.")
    except FileNotFoundError:
        print(f"File {save_path} not found.")
    except Exception as e:
        print(f"Error deleting file: {e}")

if __name__ == '__main__':
    account = sys.argv[1]
    get_code(account)
